/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.activiti.test.processinstance;

import org.activiti.ActivitiException;
import org.activiti.ProcessInstanceQuery;
import org.activiti.test.ActivitiTestCase;


/**
 * @author Joram Barrez
 */
public class ProcessInstanceQueryTest extends ActivitiTestCase {
  
  private static String PROCESS_KEY = "oneTaskProcess";
  
  private static String PROCESS_KEY_2 = "oneTaskProcess2";
  
  @Override
  protected void setUp() throws Exception {
    super.setUp();
    
    deployProcessResource("org/activiti/test/processinstance/oneTaskProcess.bpmn20.xml");
    deployProcessResource("org/activiti/test/processinstance/oneTaskProcess2.bpmn20.xml");
    
    for (int i=0; i<4; i++) {
      processService.startProcessInstanceByKey(PROCESS_KEY);      
    }
    processService.startProcessInstanceByKey(PROCESS_KEY_2);
  }

  
  public void testQueryNoSpecifics() {
    ProcessInstanceQuery query = processService.createProcessInstanceQuery();
    assertEquals(5, query.count());
    assertEquals(5, query.list().size());
    try {
      query.singleResult();
      fail();
    } catch (ActivitiException e) {}
  }
  
  public void testQueryByProcessDefinitionKey() {
    ProcessInstanceQuery query = processService.createProcessInstanceQuery().processDefinitionKey(PROCESS_KEY);
    assertEquals(4, query.count());
    assertEquals(4, query.list().size());
    try {
      query.singleResult();
      fail();
    } catch (ActivitiException e) {}
    
    query = processService.createProcessInstanceQuery().processDefinitionKey(PROCESS_KEY_2);
    assertEquals(1, query.count());
    assertEquals(1, query.list().size());
    assertNotNull(query.singleResult());
  }

}
